/*
 * blockmap.h:
 * Maintains a map of the blocks on the disk.
 * If you register a bpf, can also be used with clusters.
 * - Allows mathematical operations on the map.
 * - allows individual blocks to be annotated with a 32-bit mask.
 * - allows assigning of a name to a block or collection of blocks.
 * Optionally saves the map in the file.
 * 
 */

#ifndef BLOCKMAP_H
#define BLOCKMAP_H

#include "ftypes.h"
#include <stdio.h>

class blockmap {
private:
    typedef unsigned char dbtype;
    enum { blockmap_version=1
    };
    unsigned char *map;
    unsigned int numblocks;
    class fat_BPB *bpb;
    class blockmap *bmap;
public:
    blockmap();				// create an object
    ~blockmap();
    void set_bpb(class fat_BPB *bpb_);
    void set_bmap(class blockmap *blockmap_);

    int blocks(){return numblocks;}	// returns number of blocks in map

    void init();			// clears the blockmap
    void init(unsigned int blocks);		// initialize with # blocks

    int write(FILE *outfile);		// returns 0 if success
    int write(const char *fname);	// overwrites fname; returns 0 if success
    int read(FILE *infile);		// returns 0 if success
    int read(const char *fname);	// returns 0 if success

    blockmap & operator[](int block);

    // Bit-level annotation values
    //
    void annotate(unsigned long blocknum,char value); // set block to a bit value
    unsigned long  annotate(class ffs_chain &chain,char value); // set all blocks in chain to a bit value
                                                                // returns the actual number set

    unsigned char annotation(unsigned long blocknum); // get a bit value
    void set(unsigned long blocknum,char mask); // sets the bits in mask
    void reset(unsigned long blocknum,char mask); // resets the bits in mask
    int  test(unsigned long blocknum,char mask); // tests the bits in mask

    // Dealing with sectorchains
    int  union_count(class ffs_chain &chain,char mask); // returns number of blocks in chain with bit value set

    ulong used();				// return the number of blocks that are used
    ulong unused();			// return the number that are not used

};
#endif
