/*
 * fat_BPB.h:
 * (C) 2003 Massachusetts Institute of Technology.
 */


class fat_BPB {
    friend class ffs;
public:;
    fat_BPB();

    /* On-disk structure */
    struct BPB {
	uchar BS_jmpBoot[3]; // Jump instruction
	uchar BS_OEMName[8];
	uchar BPB_BytsPerSec[2]; // bytes per sector		
	uchar BPB_SecPerClus[1]; // sectors per cluster
	uchar BPB_RsvdSecCnt[2]; // number of reserved sectors in reserved region
	uchar BPB_NumFATs[1];    // number of FATs; usually 2
	uchar BPB_RootEntCnt[2]; // for fat12 & 16, number of root entries; 
	uchar BPB_TotSec16[2];   // for fat12 & 16, number of sectors
	uchar BPB_Media[1];      // Media code
	uchar BPB_FATSz16[2];
	uchar BPB_SecPerTrk[2];  // sectors per track, used for int 13.
	uchar BPB_NumHeads[2];   // Number of heads.
	uchar BPB_HiddSec[4];
	uchar BPB_TotSec32[4];   // Number of 32-bit sectors
	union {
	    struct fat16 {
		uchar BS_DrvNum[1]; // int13 drive number
		uchar BS_Reserved1[1];
		uchar BS_BootSig[1];
		uchar BS_VolID[4];
		uchar BS_VolLab[11];
		uchar BS_FilSysType[8];
	    } fat16;
	    
	    struct fat32 {
		uchar BPB_FATSz32[4];
		uchar BPB_ExtFlags[2];
		uchar BPB_FSVer[2];
		uchar BPB_RootClus[4];
		uchar BPB_FSInfo[2];
		uchar BPB_BkBootSec[2];
		uchar BPB_Reserved[12];
		uchar BS_DrvNum[1];
		uchar BS_Reserved1[1];
		uchar BS_BootSig[1];
		uchar BS_VolID[4];
		uchar BS_VolLab[11];
		uchar BS_FilSysType[8];
	    } fat32;
	}u;
    };

    /* Service routines */
    void   print(FILE *out);
    bool   readbuf(const uchar *buf);	// try to read a buffer.
    void   set_image_offset(long delta);
    long   image_offset;		// blocks added or subtracted from cluster to get block number
    sector_t cluster2sector(cluster_t N);

    /* Abstracted informaton */
    char BS_OEMName[9];			/* null terminated */
    int  BPB_BytsPerSec;	        // bytes per sector		
    int  BPB_SecPerClus;		// sectors per cluster
    int  BPB_RsvdSecCnt;                // number of reserved sectors in reserved region
    int  BPB_NumFATs;			// number of FATs; usually 2
    int  BPB_RootEntCnt;                // for fat12 & 16, number of root entries; 
    int  BPB_TotSec16;		        // for fat12 & 16, number of sectors
    int  BPB_TotSec32;			// Number of 32-bit sectors
    int  BPB_Media;			// Media code
    int  BPB_FATSz16;
    int  BPB_FATSz32;
    int  BPB_SecPerTrk;  // sectors per track, used for int 13.
    int  BPB_NumHeads;   // Number of heads.
    int  BPB_HiddSec;
    unsigned int CountofClusters;

    /* Fat12 & fat16 */
    int  BS_DrvNum; // int13 drive number
    int  BS_Reserved1;
    int  BS_BootSig;
    int  BS_VolID;
    char BS_VolLab[12];
    char BS_FilSysType[9];
    
    /* FAT32 */
    int  BPB_ExtFlags;
    int  BPB_FSVer;
    int  BPB_RootClus;
    int  BPB_FSInfo;
    int  BPB_BkBootSec;
    int  BPB_Reserved;
    
    /* Derrived information */
    unsigned int TotSec;		// actual total sectors
    unsigned int RootDirSectors;	// number of sectors in root directory
    int  FATType;			/* should be 12, 16 or 32 */
    int  FATSz;
    int  DataSec;			// number of data sectors
    int  FirstDataSector;
};




