/*
 * pos.cpp:
 * 
 * 
 * Copyright (c) 2002, Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * Copyright (C) 2002 Simson L. Garfinkel
 * All rights reserved.
 *
 * "A FAT file system volume is composed of four basic regions,
 * which are laid out in this order on the volume:
 * 0 - Reserved Region
 * 1 - FAT Region
 * 2 -  Root Directory Region (doesn't exist on FAT32 volumes)
 * 3 - File and Directory Data Region"
 */

/* Boot Sector and BPB Structure */
/*
 * Microsoft code follows
 */

#include "StdAfx.h"
#include "ffs.h"


pos::pos(unsigned long sector)
{
    is_sector_ = true;
    sector_  = sector;
    cluster_ = 0;
    offset_  = 0;
}
    
pos::pos(unsigned long cluster,unsigned short offset)
{
    is_sector_ = false;
    sector_  = 0;
    cluster_ = cluster;
    offset_  = offset;
}
    
sector_t pos::sector(class fat_BPB *bpb)
{
    if(is_sector_) return sector_;
    return bpb->cluster2sector(cluster_) + offset_/512;
}

unsigned long pos::cluster()
{
    return cluster_;
}

void pos::fprint(FILE *out)
{
    fprintf(out,"pos(%x) is_sector_=%d  sector_=%d cluster_=%d offset_=%d\n",
	    this,is_sector_,sector_,cluster_,offset_);
}

